// prefs.C
//
// This program is free software. See the file COPYING for details.
// Author: Mattias Engdegrd, 1997-1999

#include "prefs.h"
#include "qps.h"
#include <QLabel>
#include <QVBoxLayout>
#include <QGroupBox>
#include <qstring.h>
#include <QtGui>

#include "global.h"


#include <qstylefactory.h>
#include <qapplication.h>
#include <qstyle.h>


// class that validates input for the "swap warning limit" field
class Swapvalid : public QValidator
{
public:
    Swapvalid(QWidget *parent) : QValidator(parent) {}
    virtual State validate(QString &s, int &)const;
};

struct Boxvar
{
    const char *text;
    bool *variable;
    //CrossBox *cb;
    QCheckBox *cb;
};

static Boxvar general_boxes[] = {
   // {"Graphic Load Display", &Qps::show_load_graph, 0},
   // {"Graphic CPU Display", &Qps::show_cpu_bar, 0},
   // {"Graphic Memory Display", &Qps::show_mem_bar, 0},
   // {"Graphic Swap Display", &Qps::show_swap_bar, 0},
    //{"Minimized on Close Button", &Qps::flag_systray, 0},
    {"Exit On Close Button", &Qps::flag_exit, 0},
  //  {"Load Graph in Icon", &Qps::load_in_icon, 0},
    {"Selection: Copy PIDs to Clipboard", &Qps::pids_to_selection, 0},
  //  {"show underdevelopment ", &Qps::flag_devel, 0},
  //   {"Vertical CPU Bar (under development)", &Qps::vertical_cpu_bar, 0},
    {0, 0, 0}
};

#ifdef LINUX
static Boxvar sockinfo_boxes[] = {
    {"Host Name Lookup", &Qps::hostname_lookup, 0},
    {"Service Name Lookup", &Qps::service_lookup, 0},
    {0, 0, 0}
};
#endif

static Boxvar tree_boxes[] = {
    {"Disclosure Triangles", &Qps::tree_gadgets, 0},
    {"Branch Lines", &Qps::tree_lines, 0},
    {0, 0, 0}
};

static Boxvar misc_boxes[] = {
    {"Auto Save Settings on Exit", &Qps::auto_save_options, 0},
    {"Selection: Copy PIDs to Clipboard", &Qps::pids_to_selection, 0},
#ifdef SOLARIS
    {"Normalize NICE", &Qps::normalize_nice, 0},
    {"Use pmap for Map Names", &Qps::use_pmap, 0},
#endif
    {0, 0, 0}
};

struct Cbgroup
{
    const char *caption;
    Boxvar *boxvar;
};

static Cbgroup groups[] = {
    {"General", general_boxes},
#ifdef LINUX
//    {"Socket Info Window", sockinfo_boxes},
#endif
//   {"Tree View", tree_boxes},
//    {"Miscellaneous", misc_boxes},
    {0, 0}
};

struct fontsets {
    const char *set;
    //char *name;
    QString name;
    int size;
    int avail;
};


//static 
fontsets  recommend_fontset[] = {
    {"System Default", "system_default",10,0},
    {"screen, 10 point (Solaris)","screen",10,0},  // non-antialis !
    {"Luxi Sans, 9 point", "Luxi Sans",9,0},
    {"Bitstream Vera Sans, 9 point","Bitstream Vera Sans",9,0},
    {"Bitstream Vera Sans Mono, 9 point","Bitstream Vera Sans Mono",9,0},
    {"Courier 10 Pitch, 9 point","Courier 10 Pitch",9,0},
    {"Verdana, 9 point","Verdana",9,0},
    {"LucidaTypewriter, 10 point","LucidaTypewriter",10,0},  // non-anti-alias
    {"Lucida, 10 point","Lucida",10,0},			// non-antialias
    {"Helvetica, 10 point","Helvetica",10,0},	// non-anti
    {0,0,0,0}
};

void getDefaultFont(QFont f)
{
	recommend_fontset[0].name=f.family();
	recommend_fontset[0].size=f.pointSize();
}

void find_fontsets();
// dual use function: both validate and apply changes
QValidator::State Swapvalid::validate(QString &s, int &) const
{
	// only accept /^[0-9]*[%kKmM]?$/
	int len = s.length();
	int i = 0;
	while(i < len && s[i] >= '0' && s[i] <= '9') i++;
	if(i < len && QString("kKmM%").contains(s[i]) == 0 || i < len - 1)
		return Invalid;
	if(s[i] == 'k')
		s[i] = 'K';
	if(s[i] == 'm')
		s[i] = 'M';
	//int val = atoi((const char *)s);
	int val = s.toInt();
	bool percent;
	if(s[i] == '%') {
		percent = TRUE;
	} else {
		percent = FALSE;
		if(s[i] == 'M')
			val <<= 10;
	}
	Qps::swaplimit = val;
	Qps::swaplim_percent = percent;
	return Acceptable;
}


Preferences::Preferences(): QDialog()
{
	int flag_test=0;
    setWindowTitle("Preferences");
	QVBoxLayout *v_layout = new QVBoxLayout;

	if(flag_test)
	{	
		QTabWidget *tbar = new QTabWidget(this);
		QWidget *w=new QWidget(this);
		tbar->addTab(w, "&Setting");
		w->setLayout(v_layout);
	}
	else
		setLayout(v_layout);
	
    v_layout->setSpacing(1);
    //v_layout->setSpacing(1);
	const int border_x = 10;
    int min_x = 0;

    for(Cbgroup *g = groups; g->caption; g++) {
        QGroupBox *grp = new QGroupBox(g->caption, this);
     	QVBoxLayout *vbox = new QVBoxLayout;
		for(Boxvar *b = g->boxvar; b->text; b++) 
        {
            b->cb = new QCheckBox(b->text, grp);
     		vbox->addWidget(b->cb);
            connect(b->cb, SIGNAL(clicked()), SLOT(update_reality())); 
            // -> EMIT prefs_change()
        }
		grp->setLayout(vbox);
        v_layout->addWidget(grp);
    }
    update_boxes();

	/*
	QGroupBox *wgrp = new QGroupBox("Swap Warning", this);
    QHBoxLayout *hl = new QHBoxLayout;
	wgrp->setLayout(hl);
	QLabel *lbl = new QLabel("Warn when free swap below", wgrp);
	QLineEdit *swaped = new QLineEdit(wgrp);
    hl->addWidget(lbl);
    hl->addWidget(swaped);
	//swaped->setAlignment (  Qt::AlignRight );
	QString s;
	if(Qps::swaplim_percent) {
		s.sprintf("%d%%", Qps::swaplimit);
	} else {
		if(Qps::swaplimit >= 1024 && (Qps::swaplimit & 0x7ff) == 0)
			s.sprintf("%dM", Qps::swaplimit >> 10);
		else
			s.sprintf("%dK", Qps::swaplimit);
	}
	swaped->setText(s);
	swaped->setValidator(new Swapvalid(this));
	connect(swaped, SIGNAL(textChanged(const QString &)), SIGNAL(prefs_change()));
	v_layout->addWidget(wgrp);
	*/

	// Appearance ====================================
	if(1){
		QGroupBox *gbox = new QGroupBox("Appearance", this);
		QVBoxLayout *vbox = new QVBoxLayout;
		QHBoxLayout *hbox=new QHBoxLayout();
    	font_cb=new QFontComboBox(this);
		font_cb->setWritingSystem ( QFontDatabase::Latin );
		psizecombo = new QComboBox(this);
		//hbox->addWidget(familycombo);
		hbox->addWidget(font_cb);
		hbox->addWidget(psizecombo);
    	vbox->addLayout(hbox);
		gbox->setLayout(vbox);
		v_layout->addWidget(gbox);
		//connect(fcb, SIGNAL(currentFontChanged (const QFont & font )),this, SLOT(font_changed(int)));
		connect(font_cb, SIGNAL(activated( int)),this, SLOT(font_changed(int)));
		connect(psizecombo, SIGNAL(activated ( int )), SLOT(font_changed(int )));
	
	// add to font size 	
		init_font_size();
	}
	if(0){
		QGroupBox *font_grp = new QGroupBox("Appearance", this);
		QVBoxLayout *vbox = new QVBoxLayout;
		QLabel *label1 = new QLabel("Current Font", font_grp);
		QHBoxLayout *hbox=new QHBoxLayout();
		//hbox->setSpacing (1);
		familycombo = new QComboBox(font_grp);
		psizecombo = new QComboBox(font_grp);
		hbox->addWidget(familycombo);
		hbox->addWidget(psizecombo);
    	vbox->addWidget(label1);
    	vbox->addLayout(hbox);

		QLabel *label2 = new QLabel("Recommended Font Set ", font_grp);
		fontset_combo = new QComboBox(font_grp);
    	vbox->addWidget(label2);
    	vbox->addWidget(fontset_combo);
		font_grp->setLayout(vbox);
	//v_layout->addWidget(font_grp);
	
	connect(familycombo, SIGNAL(activated( int)),this, SLOT(font_changed(int)));
	connect(psizecombo, SIGNAL(activated ( int )), SLOT(font_changed(int )));
	connect(fontset_combo, SIGNAL(activated ( int )), SLOT(fontset_changed(int )));
	}

	/*
	// Style, Themes ==================================
	QLabel *label3 = new QLabel("Themes", font_grp);	
	QComboBox *theme_combo = new QComboBox(font_grp);

	QStringList styles = QStyleFactory::keys();
    styles.sort();
    theme_combo->insertStringList(styles);

	QString currentstyle;
	currentstyle = QApplication::style().className();
*/
	

	QPushButton *saveButton = new QPushButton("OK", this);
   // saveButton->setFocusPolicy(QWidget::NoFocus);
    saveButton->setFocus();
	saveButton->setDefault(TRUE);
	v_layout->addWidget(saveButton);
	//v_layout->freeze();
	
	connect(saveButton, SIGNAL(clicked()), SLOT(closed()));
	return;

	for(int i=0;recommend_fontset[i].size!=0;i++)
	{
		if(recommend_fontset[i].avail>0)
			fontset_combo->insertItem(i,recommend_fontset[i].set);
	}

}

void removeFromQStringList(QStringList &a,QStringList &b)
{
	QStringList::Iterator sit = b.begin();
/*	
	while (sit != b.end()) {
		QStringList::Iterator si=a.indexOf(*sit);
		a.remove(si);
		sit++;
	}
*/
}

void removeFromQStringListGrep(QStringList &a,QStringList &b)
{
	QStringList::Iterator sit = b.begin();
/*	
	while (sit != b.end()) {
		QStringList c=a.grep(*sit,false);
		removeFromQStringList(a,c);	
		sit++;
	}
*/
}

void addsFromQStringListGrep(QStringList &a,QStringList &b)
{
 	QFontDatabase db;
	QStringList families = db.families(QFontDatabase::Latin); 
	QStringList::Iterator sit = b.begin();
	QStringList c;

	/*
	while (sit != b.end()) {
		QStringList c=a.grep(*sit,false);
		removeFromQStringList(a,c);	
		sit++;
	}*/
}




// DRAFT CODE : 
// 1.update font list
// 2.find current_default font name and Index
// 
void Preferences::init_font_size()
{
 	QFontDatabase db;
	///QStringList families ( WritingSystem writingSystem = Any ) const	
	QStringList families = db.families(QFontDatabase::Latin);
	QStringList extra;
	/*
	for(int i=0;recommend_fontset[i].size!=0;i++)
	{
		//QStringList l=families.grep(recommend_fontset[i].name,false);
		//if(l.size()>0)
		if(families.contains(recommend_fontset[i].name,Qt::CaseInsensitive))	
		{
			recommend_fontset[i].avail=1;
		}

	} */
	
	/* 
 *  familycombo->clear();
	familycombo->insertItems(0,families);
	// find the default font's family
	QStringList::Iterator sit = families.begin();
	int i = 0, possible = -1, font_idx=-1;
	while (sit != families.end()) {
		if (*sit == QApplication::font().family())
		{
			font_idx=i;
			break;
		}
		if ((*sit).contains(QApplication::font().family()))
			possible = i;
		i++;
		sit++;
	}

	if (font_idx==-1)
		i = possible;
	if (i == -1) // no clue about the current font
		i = 0;

	familycombo->setCurrentIndex(i);
	

*/
	psizecombo->clear();

	
	QList<int> sizes = db.standardSizes();
	QList<int>::Iterator it = sizes.begin();
	int i,idx=0;
	while (it != sizes.end())
	{	
		if( *it <24 && *it > 5 )	
			psizecombo->insertItem(idx++,QString::number(*it++));
		else it++;
	}
	
	
	// find current font size
	i = 0;
	for (int psize = QApplication::font().pointSize(); i < psizecombo->count(); ++i) 
	{
		const int sz = psizecombo->itemText(i).toInt();
		if (sz == psize) {
			psizecombo->setCurrentIndex(i);
			break;
		}
	}
}


// slot: update check boxes to reflect current status
void Preferences::update_boxes()
{
    for(Cbgroup *g = groups; g->caption; g++)
	for(Boxvar *b = g->boxvar; b->text; b++)
	    b->cb->setChecked(*b->variable);
}

// slot: update flags and repaint to reflect state of check boxes
void Preferences::update_reality()
{
	for(Cbgroup *g = groups; g->caption; g++)
		for(Boxvar *b = g->boxvar; b->text; b++)
			*b->variable = b->cb->isChecked();
	emit prefs_change();
}

void Preferences::closed()
{
   	hide();
	qps->write_settings();
}

// work
void Preferences::font_changed(int i)
{
	int size=psizecombo->currentText().toInt();
	QFont font=font_cb->currentFont();
	font.setPointSize(size);
	
 	QApplication::setFont(font);
	qps->font_name=font.family();
	qps->font_size=size;
}

//DRAFT CODE:
void Preferences::fontset_changed(int i)
{
	return; 
	QString fontset=fontset_combo->currentText();//key_escape;
	
	for(i=0;recommend_fontset[i].size!=0;i++)
	{
		if( fontset==recommend_fontset[i].set)
		{
			QFont font(recommend_fontset[i].name,recommend_fontset[i].size);
	 		QApplication::setFont(font);
			//remove_bad_fonts();
			font_changed(0);
		}
	}
}
